@@ -4,19 +4,28 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 from loguru import logger
-from .config import Style
-from .utils import PickleUtils, Rosetta
-from .data_quality import basis_quality_checks, basis_quality_plots, fill_ratio
+from src.isthmuslib.config import Style
+from src.isthmuslib.utils import PickleUtils, Rosetta, make_dict
+from src.isthmuslib.data_quality import basis_quality_checks, basis_quality_plots, fill_ratio
 from copy import deepcopy
 import statsmodels.api as sm
-from .plotting import visualize_x_y, visualize_1d_distribution, visualize_surface
+from .plotting import visualize_x_y, visualize_1d_distribution, visualize_surface, visualize_embedded_surface
 import pathlib
+from pydantic import BaseModel
+from sklearn.feature_selection import SelectKBest, chi2
+
+
+class SVD(BaseModel):
+    u: Any
+    s: Any
+    vh: Any
 
 
 class VectorMultiset(PickleUtils, Style, Rosetta):
     """ A set of vectors (which may or may not be ordered)"""
     data: pd.DataFrame = None
     name_root: str = None
+    svd: SVD = None
 
     class Config:
         arbitrary_types_allowed = True
@@ -170,6 +179,15 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         kwargs.setdefault('title', self.translate(z_name, missing_response='return_input'))
         return visualize_surface(self.data.loc[:, x_name], self.data.loc[:, y_name], self.data.loc[:, z_name], **kwargs)
 
+    def visualize_embedded_surface(self, x_col_name: str, y_col_name: str, z_col_name: str, **kwargs) -> plt.Figure:
+        """ Plots a surface (2D embedded in 3D) based on vector features"""
+        return visualize_embedded_surface(self.data.loc[:, x_col_name], self.data.loc[:, y_col_name],
+                                          self.data.loc[:, z_col_name], **kwargs)
+
+    ################
+    # Statistics
+    ################
+
     def correlation_matrix(self, **kwargs) -> pd.DataFrame:
         """
         Very thin wrapper around correlation matrix (using pandas corr() method)
@@ -179,6 +197,69 @@ class VectorMultiset(PickleUtils, Style, Rosetta):
         """
         return correlation_matrix(self.data, **kwargs)
 
+    def singular_value_decomposition(self, cols: Union[str, List[str]] = None,
+                                     cache_results: bool = False,
+                                     **kwargs) -> SVD:
+        """ Applies singular value decomposition
+
+        :param cache_results: if True, saves u, s, and vh as the attribute 'svd' (an SVD class)
+        :param cols: which data features to use
+        :param kwargs: keyword arguments for SVD
+        :return: u, s, vh arrays
+        """
+        if not cols:
+            cols = self.data.keys().tolist()
+        svd: SVD = singular_value_decomposition(self.data, cols, **kwargs)
+        if cache_results:
+            self.svd = svd
+        return svd
+
+    def to_surface_df(self, group_by_col_names: List[str], aggregation_method: str = 'max', **kwargs) -> pd.DataFrame:
+        """ Thin wrapper that aggregates the data to a surface, using pandas methods like mean, median, max, etc
+
+        :param group_by_col_names: axes for the surface (i.e. how to bin the data)
+        :param aggregation_method: how to combine multiple data points in the same bin
+        :param kwargs: additional keyword arguments for pandas groupby method
+        :return: data frame with combined according `aggregation_method` into bins of `group_by_col_names`
+        """
+        return getattr(self.data.groupby(by=group_by_col_names, **kwargs), aggregation_method)().reset_index(drop=False)
+
+    def plot_projection_to_surface(self, surface_axes_names: Union[List[str], Tuple[str, str]], z_axis_name: str,
+                                   aggregation_method: str = 'max', **kwargs) -> plt.Figure:
+        if len(surface_axes_names) != 2:
+            raise ValueError(f"plot_projection is intended to be used with 2D surfaces")
+        df_surface: pd.DataFrame = self.to_surface_df(group_by_col_names=surface_axes_names,
+                                                      aggregation_method=aggregation_method,
+                                                      **kwargs)
+        return visualize_x_y(df_surface.loc[:, surface_axes_names[0]],
+                             df_surface.loc[:, surface_axes_names[1]],
+                             c=df_surface.loc[:, z_axis_name].tolist(),
+                             xlabel=self.translate(surface_axes_names[0]),
+                             ylabel=self.translate(surface_axes_names[1]),
+                             colorbar_label=self.translate(z_axis_name))
+
+    def feature_selection_univariate(self, target_feature_name: str, input_feature_names: List[str] = None,
+                                     k_best: int = 3, normalize: bool = True, **kwargs) -> any:
+        """ Feature selection of k best using univariate methods.
+
+        :param target_feature_name: column name of the target feature
+        :param input_feature_names: column names of the input features
+        :param k_best: how many features to return
+        :param normalize: if True, scales input features to unit mean
+        :param kwargs: additional keyword arguments passed through to scikit-learn SelectKBest
+        :return: trimmed input feature data set
+        """
+        if not input_feature_names:
+            input_feature_names = [x for x in self.data.keys() if x != target_feature_name]
+        input_feature_data = self.data.loc[:, input_feature_names]
+        if normalize:
+            for fieldname in input_feature_data.keys():
+                std_dev: float = float(np.std(this_field_data := self.data.loc[:, fieldname]))
+                input_feature_data[fieldname] = [x / std_dev for x in this_field_data]
+
+        target_feature_data = self.data.loc[:, target_feature_name].to_numpy()
+        return SelectKBest(chi2, k=k_best).fit_transform(input_feature_data, target_feature_data, **kwargs)
+
 
 class SlidingWindowResults(VectorMultiset):
     """ Results from Sequence.sliding_window(), which is a VectorMultiset with extra context baked in """
@@ -388,7 +469,7 @@ class VectorSequence(VectorMultiset):
             # First, figure out where to place the windows:
             if not window_starts:
                 if step_size:
-                    window_starts: List[float] = list(np.arange(min(basis), max(basis) - step_size, step_size))
+                    window_starts: List[float] = list(np.arange(min(basis), max(basis) - window_width, step_size))  # !
                 else:
                     if overlapping:
                         window_starts: List[float] = [x for x in basis if basis[-1] - x >= window_width]
@@ -401,8 +482,8 @@ class VectorSequence(VectorMultiset):
                 window_data: VectorSequence = self.slice(start_at=window_start, stop_at=window_start + window_width,
                                                          inplace=False, reset_index=True)
                 evaluation: Dict[str, Any] = function(window_data, *args, **kwargs)
-                df = df.append({'window_width': window_width, 'window_start': window_start,
-                                **evaluation}, ignore_index=True)
+                new_entry: Dict[Any, Any] = {'window_width': window_width, 'window_start': window_start, **evaluation}
+                df = pd.concat([df, pd.DataFrame(new_entry, index=[0])], ignore_index=True)
 
                 if verbose:
                     logger.info(f"\nWindow width: {window_width} ({i + 1} of {len(window_widths)})\n"
@@ -499,17 +580,21 @@ class VectorSequence(VectorMultiset):
 
         return figure_handles
 
-    def singular_value_decomposition(self, cols: Union[str, List[str]] = None,
-                                     **kwargs) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
-        """ Helper function that wraps numpy svd to feed in a subset of data features (see kwarg: 'full_matrices')
+    def singular_value_decomposition(self, cols: Union[str, List[str]] = None, cache_results: bool = False,
+                                     **kwargs) -> SVD:
+        """ Applies singular value decomposition, skipping the basis column
 
+        :param cache_results: if True, saves u, s, and vh as the attribute 'svd' (an SVD class)
         :param cols: which data features to use
         :param kwargs: keyword arguments for SVD
         :return: u, s, vh arrays
         """
         if not cols:
             cols = [x for x in self.data.keys().tolist() if x != self.basis_col_name]
-        return np.linalg.svd(deepcopy(self.data.loc[:, cols]), **kwargs)
+        svd: SVD = singular_value_decomposition(self.data, cols, **kwargs)
+        if cache_results:
+            self.svd = svd
+        return svd
 
     def calculate_info_surface(self, window_widths: List[float] = None, cols: Union[str, List[str]] = None, *args,
                                **kwargs) -> InfoSurface:
@@ -538,12 +623,13 @@ class VectorSequence(VectorMultiset):
         :return: list of figure handles
         """
         # Set style. Overrides: kwargs > style input > Style() defaults
-        config: Style = Style(**Style().override(style).dict() | kwargs)
-        svd_kwargs: dict[str, Any] = {k: v for k, v in kwargs.items() if k not in config.dict()}
-        style_kwargs: dict[str, Any] = {k: v for k, v in kwargs.items() if k in config.dict()}
+
+        config: Style = Style(**{**Style().dict(), **make_dict(style), **make_dict(kwargs)})
+        svd_kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if k not in config.dict()}
+        style_kwargs: Dict[str, Any] = {k: v for k, v in kwargs.items() if k in config.dict()}
 
         result: InfoSurface = self.calculate_info_surface(window_widths=window_widths, cols=cols, *args, **svd_kwargs)
-        return result.plot_info_surface(singular_values=singular_values, **style_kwargs)
+        return result.plot_info_surface(singular_values=singular_values, **{**make_dict(style), **make_dict(kwargs)})
 
     def correlation_matrix(self, exclude_basis: bool = True, **kwargs) -> pd.DataFrame:
         """
@@ -558,11 +644,15 @@ class VectorSequence(VectorMultiset):
                 if self.basis_col_name not in kwargs_in:
                     kwargs['exclude_cols'] = kwargs_in + [self.basis_col_name]
             else:
-                kwargs: dict[str, Any] = {'exclude_cols': self.basis_col_name}
+                kwargs: Dict[str, Any] = {'exclude_cols': self.basis_col_name}
         return correlation_matrix(self.data, **kwargs)
 
+    ################
+    # Statistics
+    ################
 
-def correlation_matrix(dataframe: pd.DataFrame, use_cols: list[str] = None, exclude_cols: list[str] = None,
+
+def correlation_matrix(dataframe: pd.DataFrame, use_cols: List[str] = None, exclude_cols: List[str] = None,
                        correlation_method: str = 'pearson', style: Style = None, **kwargs) -> pd.DataFrame:
     """
     Returns a styled pandas data frame with correlation coefficients (Pearson by default), wraps pandas.corr()
@@ -576,9 +666,9 @@ def correlation_matrix(dataframe: pd.DataFrame, use_cols: list[str] = None, excl
     :return: styled pandas dataframe
     """
     if not use_cols:
-        use_cols: list[str] = dataframe.keys().tolist()
+        use_cols: List[str] = dataframe.keys().tolist()
     if exclude_cols:
-        use_cols: list[str] = [x for x in use_cols if x not in exclude_cols]
+        use_cols: List[str] = [x for x in use_cols if x not in exclude_cols]
     if style:
         kwargs.setdefault('cmap', style.cmap)
     else:
@@ -595,8 +685,20 @@ def info_surface_slider(vs: VectorSequence, *args, **kwargs) -> Dict[str, Any]:
     :param kwargs: kwargs for svd
     :return: dictionary with singular values
     """
-    _, s, _ = vs.singular_value_decomposition(*args, **kwargs)
-    return {f"singular_value_{i + 1}": value for i, value in enumerate(s)}
+    svd: SVD = vs.singular_value_decomposition(*args, **kwargs)
+    return {f"singular_value_{i + 1}": value for i, value in enumerate(svd.s)}
+
+
+def singular_value_decomposition(df: pd.DataFrame, cols: Union[str, List[str]] = None, **kwargs) -> SVD:
+    """ Helper function that wraps numpy svd to feed in a subset of data features (see kwarg: 'full_matrices')
+
+    :param df: pandas dataframe to analyze
+    :param cols: which data features to use
+    :param kwargs: keyword arguments for SVD
+    :return: u, s, vh arrays
+    """
+    u, s, vh = np.linalg.svd(deepcopy(df.loc[:, cols]), **kwargs)
+    return SVD(u=u, s=s, vh=vh)
 
 
 class OrderedSeries:
